"""
Help Viewer

TODO: use wx.html2

"""

import inspect
import logging
import types
import webbrowser

import wx.html
import docutils.core

from .. import fmtxt
from .._wxutils import Icon, ID



HtmlTemplate = """<pre class="literal-block">%s</pre>"""


def rst2html(rst):
    """
    reStructuredText to HTML parsing following:
    http://stackoverflow.com/questions/6654519
    """
    # remove leading whitespaces; make an exception for the first line,
    # since several functions start their docstring on the first line
    rst = fmtxt.unindent(rst, True)

    try:
        html = docutils.core.publish_parts(rst, writer_name='html')['body']
        if "ERROR/3" in html:
            raise RuntimeError("rst2html unresolved cross-ref")
        logging.debug("rst2html success")
#            html = os.linesep.join((html['stylesheet'], html['body']))
#            html = html['whole']
#            html = '<span style="color: rgb(0, 0, 255);">RST2HTML:</span><br>' + html
    except:
        html = HtmlTemplate % rst
        logging.debug("rst2html failed")
#            html = '<span style="color: rgb(255, 0, 0);">RST2HTML FAILED:</span><br>' + html
    return html


def doc2html(obj, default='No doc-string.<br>'):
    """
    Returns the object's docstring as html, or the default value if there is
    no docstring.

    """
    if hasattr(obj, '__doc__') and isinstance(obj.__doc__, basestring):
        txt = rst2html(obj.__doc__)
    else:
        txt = default
    return txt




def format_chapter(title, txt):
    return '\n'.join(('\n',
                      '-' * 80,
                      title + ' |',
                      '-' * (2 + len(title)),
                      txt, '\n'))

def format_subtitle(subtitle):
    return '<span style="color: rgb(102, 102, 102);">%s</span>' % subtitle


class HelpFrame(wx.Frame):
    def __init__(self, parent, *args, **kwargs):
        wx.Frame.__init__(self, parent, *args, **kwargs)
        self.EnableCloseButton(False)
        self.parent_shell = parent

        self.help_formatter = HTMLHelpFormatter()

        style = wx.NO_FULL_REPAINT_ON_RESIZE
        self.HTMLPanel = wx.html.HtmlWindow(self, style=style)
        self.HTMLPanel.Bind(wx.html.EVT_HTML_LINK_CLICKED, self.OnLinkClicked)

#        html.SetRelatedFrame(parent)
        # after wxPython Demo
        if "gtk2" in wx.PlatformInfo:
            self.SetStandardFonts()


        # prepare data container
        self.history = []
        self.current_history_id = -1

        # TOOLBAR ---
        self.toolbar = tb = self.CreateToolBar(wx.TB_HORIZONTAL)
        tb.SetToolBitmapSize(size=(32, 32))

        # hide
        tb.AddLabelTool(ID.HELP_HIDE, "Hide", Icon("tango/status/image-missing"))
        self.Bind(wx.EVT_TOOL, self.OnHide, id=ID.HELP_HIDE)
        tb.AddSeparator()

        # forward/backward
        tb.AddLabelTool(wx.ID_HOME, "Home", Icon("tango/places/start-here"))
        self.Bind(wx.EVT_TOOL, self.OnHome, id=wx.ID_HOME)
        tb.AddLabelTool(wx.ID_BACKWARD, "Back", Icon("tango/actions/go-previous"))
        self.Bind(wx.EVT_TOOL, self.OnBackward, id=wx.ID_BACKWARD)
        tb.AddLabelTool(wx.ID_FORWARD, "Next", Icon("tango/actions/go-next"))
        self.Bind(wx.EVT_TOOL, self.OnForward, id=wx.ID_FORWARD)
        tb.EnableTool(wx.ID_FORWARD, False)
        tb.EnableTool(wx.ID_BACKWARD, False)

        # text search
        self.history_menu = wx.Menu()
        item = self.history_menu.Append(-1, "Help History")
        item.Enable(False)
        search_ctrl = wx.SearchCtrl(tb, wx.ID_HELP, style=wx.TE_PROCESS_ENTER,
                                    size=(300, -1))
        search_ctrl.Bind(wx.EVT_TEXT_ENTER, self.OnSelfSearch)
        search_ctrl.Bind(wx.EVT_SEARCHCTRL_SEARCH_BTN, self.OnSelfSearch)
        search_ctrl.SetMenu(self.history_menu)
        self.history_menu.Bind(wx.EVT_MENU, self.OnSearchhistory)
        search_ctrl.ShowCancelButton(True)
        self.Bind(wx.EVT_SEARCHCTRL_CANCEL_BTN, self.OnSearchCancel, search_ctrl)
        tb.AddControl(search_ctrl)
        self.search_ctrl = search_ctrl


        if wx.__version__ >= '2.9':
            tb.AddStretchableSpace()
        else:
            tb.AddSeparator()

        # clear cache
        tb.AddLabelTool(ID.CLEAR_CACHE, "Clear Cache", Icon("tango/actions/edit-clear"))
        self.Bind(wx.EVT_TOOL, self.OnClearCache, id=ID.CLEAR_CACHE)

        # finish
        tb.Realize()

    def display(self, name):
        if name in self.help_formatter.help_items:
            content = self.help_formatter.help_items[name]
            self.HTMLPanel.SetPage(content)
        else:
            raise ValueError("No help object for %r" % name)

        self.search_ctrl.SetValue(name)
        self.SetTitle("Help: %s" % name)
        self.Show()

    def HelpLookup(self, topic=None, name=None):
        """
        Display help for a topic. Topic can be
         - None -> display default help
         - an object -> display help for the object based on its doc-string

        """
        if topic is None:
            name = 'Start Page'
        else:
            name = self.help_formatter.add_object(topic)

        self.display(name)

        if name in self.history:
            index = self.history.index(name)
            self.history.pop(index)

        i = self.current_history_id
        if (i != -1) and (len(self.history) > i + 1):
            self.history = self.history[0:i + 1]

        self.history.append(name)
        self.set_current_history_id(-1)

        self.Raise()

    def OnClearCache(self, event):
        self.help_formatter.delete_cache()

    def OnHide(self, event=None):
        self.Show(False)

    def OnHome(self, event=None):
        self.HelpLookup(topic=None)

    def OnBackward(self, event=None):
        i = self.current_history_id
        if i == -1:
            i = len(self.history) - 1

        i -= 1
        name = self.history[i]
        self.display(name)
        self.set_current_history_id(i)

    def OnForward(self, event=None):
        i = self.current_history_id + 1

        name = self.history[i]
        self.display(name)
        self.set_current_history_id(i)

    def OnLinkClicked(self, event):
        URL = event.GetLinkInfo().GetHref()
        # is there a better way to distinguish external from internal links?
        if URL.startswith('http://') or URL.startswith('www.'):
            webbrowser.open(URL)
        else:
            event.Skip()

    def OnSearchCancel(self, event=None):
        self.search_ctrl.Clear()

    def OnSearchhistory(self, event=None):
        i = event.GetId() - 1
        self.display(i)

    def OnSelfSearch(self, event=None):
        txt = event.GetString()
        if len(txt) > 0:
            self.text_lookup(txt)

    def set_current_history_id(self, i):
        self.current_history_id = i

        if i == -1:
            exists_greater = False
            exists_smaller = len(self.history) > 1
        else:
            exists_greater = len(self.history) > i + 1
            exists_smaller = i > 0
        self.toolbar.EnableTool(wx.ID_FORWARD, exists_greater)
        self.toolbar.EnableTool(wx.ID_BACKWARD, exists_smaller)

    def text_lookup(self, txt):
        logging.debug("Help text_lookup: %r" % txt)
        try:
            obj = eval(txt, self.parent_shell.global_namespace)
        except:
            dlg = wx.MessageDialog(self, "No object named %r in shell namespace" % txt,
                                   "Help Lookup Error:", wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        else:
            self.HelpLookup(obj, name=txt)


class HTMLHelpFormatter(object):
    """
    """
    def __init__(self):
        self.delete_cache()

    def add_object(self, obj, name=None):
        if not name:
            if hasattr(obj, '__name__'):
                name = obj.__name__
            elif hasattr(obj, '__class__'):
                name = obj.__class__.__name__
            else:
                raise ValueError("No Name For Help Object")

        if name not in self.help_items:
            self.help_items[name] = self.parse_object(obj)

        return name

    def delete_cache(self):
        "removes all stored help entries"
        self.help_items = {'Start Page': self.get_help_home()}

    def get_help_home(self):
        title = "PyShell"
        title = '\n'.join([title, '-' * len(title)])
        intro = "PyShell ``HELP_TEXT``::"
        pyshell_doc = '\t' + wx.py.shell.HELP_TEXT.replace('\n', '\n\t')
        text = '\n\n'.join([_main_help, title, intro, pyshell_doc])
        return self.parse_text(text)

    def parse_object(self, obj):
        """
        Parse the object's doc-string

        """
        if not hasattr(obj, '__doc__'):
            raise ValueError("Object does not have a doc-string.")

        attrs = {}
        # ## new customized parsing
        if isinstance(obj, (types.MethodType, types.BuiltinMethodType)):
            title = "%s(...)" % obj.__name__
            if isinstance(obj, types.MethodType):
                subtitle = "Method of %s" % str(type(obj.im_class))[1:-1]
            else:
                subtitle = "Method of %s" % str(type(obj.__self__))[1:-1]
            intro = doc2html(obj) + '<br>'
        elif isinstance(obj, (types.FunctionType, types.BuiltinFunctionType)):
            title = "%s(...)" % obj.__name__
            module = obj.__module__
            if module == '__builtin__':
                subtitle = "builtin function"
            else:
                subtitle = "function in %s" % module

            # function signature
            if inspect.isfunction(obj):
                a = inspect.getargspec(obj)
                if a.defaults:
                    n_args = len(a.args) - len(a.defaults)
                    args = a.args[:n_args]
                    args.extend(map('='.join, zip(a.args[n_args:], map(repr, a.defaults))))
                else:
                    args = a.args
                if a.varargs:
                    args.append('*%s' % a.varargs)
                if a.keywords:
                    args.append('**%s' % a.keywords)
                signature = "%s(%s)" % (obj.__name__, ', '.join(args))
                subtitle = '<br>'.join((subtitle, signature))

            intro = doc2html(obj)
        elif isinstance(obj, types.ObjectType):
            if isinstance(obj, (types.TypeType, types.ClassType)):
                # ClassType: old-style classes (?)
                title = "%s" % obj.__name__
                module = obj.__module__
            else:
                title = "%s" % obj.__class__.__name__
                module = obj.__class__.__module__
            if module == '__builtin__':
                subtitle = "builtin class"
            else:
                subtitle = "class in %s" % module
            intro = '<br>'.join((doc2html(obj),
                                 "<h2>Initialization</h2>",
#                                 format_subtitle("(...)"),
                                 doc2html(obj.__init__)))
            attrs_title = "Methods"
            for attr in dir(obj):
                if attr.startswith('_'):
                    continue
                a = getattr(obj, attr)
                if not hasattr(a, '__call__'):
                    continue
                attrs[attr] = "", doc2html(a)
        # ## OLD default parsing
        else:
            is_function = isinstance(obj, (types.FunctionType, types.BuiltinFunctionType))

            # doc-string for the object itself
            obj_type = str(type(obj))[1:-1]
            if hasattr(obj, '__name__'):
                title = obj.__name__
                subtitle = obj_type
            else:
                title = obj_type
                subtitle = None

            intro = doc2html(obj)
            if not is_function:
                intro += '<br><br><hr style="width: 100%; height: 2px;"><br><br>'

            # collect attributes
            if not is_function:
                attrs_title = "Attributes"
                for attr in dir(obj):
                    if attr[0] != '_' or attr == '__init__':
                        try:
                            a = getattr(obj, attr)
                        except:
                            pass
                        else:
                            typename = str(type(a))[6:-1]
                            attrs[attr] = typename, doc2html(a)

        # add text for attrs
        TOC = []
        chapters = []
        if len(attrs) > 0:
            for name in sorted(attrs):
                typename, doc = attrs[name]
                if doc:
                    TOC.append('<a href="#%s">%s</a> %s<br>' % (name, name, typename))
                    chapters.append('<h2><a href="#TOC">&uarr;</a><a name="%s"></a>%s</h2><br>%s' \
                                    % (name, name, doc))
                else:
                    TOC.append('<span style="color: rgb(102, 102, 102);">'
                               '%s (no __doc__)</span><br>' % name)

        # compose text
        txt = "<h1>%s</h1><br>" % title
        if subtitle:
            txt += '%s<br>' % format_subtitle(subtitle)
        txt += intro

        if len(TOC) > 0:
            txt += '<h1><a name="TOC"></a>%s</h1><br>' % attrs_title
            txt += ''.join(TOC)
            txt += '<br>'.join(chapters)

        if txt == '':
            txt = "Error: No doc-string found."

        return txt

    def parse_text(self, text):
        return rst2html(text)


_main_help = """
Eelbrain online documentation: https://pythonhosted.org/eelbrain/

"""




import ast
import os
import logging

import wx.stc
import wx.py
# import wx.lib.mixins.listctrl

from .. import fmtxt
from .._wxutils import droptarget, Icon, ID, key_mod


# replace Document to allow unicode i/o
from wx.py.document import Document as wxDocument
class Document(wxDocument):

    def read(self):
        text = wxDocument.read(self)
        return text

    def write(self, text):
        text = text.encode('utf-8')
        wxDocument.write(self, text)

wx.py.document.Document = Document


# fix Editor class
class Editor(wx.py.editor.Editor):

    def CallTipShow(self, pos, tip):
        self.window.CallTipShow(pos, tip)

    def GetColumn(self, pos):
        return self.window.GetColumn(pos)

wx.py.editor.Editor = Editor


class PyEditor(wx.py.editor.EditorFrame):
    """
    Editor.editor.window is based on StyledTextCtrl

    """
    def __init__(self, parent, shell, pos=(0, 0), size=(640, 840), pyfile=None,
                 name=None):
        """
        Parameters
        ----------
        pyfile : bool | str
            Filename, or True in order to display an open file dialog,
            False/None in order to open a new buffer.
        name : None | str
            Name for the new buffer.
        """
        shell.active_editor = self
        self.shell = shell
        self._exec_in_shell_namespace = True

        if isinstance(pyfile, basestring):
            filename = pyfile
        else:
            filename = None

        super(PyEditor, self).__init__(parent, title=name or "Script Editor",
                                       size=size, pos=pos, filename=filename)

    # toolbar ---
        tb = self.CreateToolBar(wx.TB_HORIZONTAL)
        tb.SetToolBitmapSize(size=(32, 32))


        # script execution
        tb.AddLabelTool(ID.EXEC_DOCUMENT, "Exec", Icon("actions/python-run"),
                        shortHelp="Execute script in shell")
        self.Bind(wx.EVT_TOOL, self.OnExecDocument, id=ID.EXEC_DOCUMENT)
        tb.AddLabelTool(ID.EXEC_SELECTION, "Exec_sel",
                        Icon("actions/python-run-selection"),
                        shortHelp="Execute selection in shell")
        self.Bind(wx.EVT_TOOL, self.OnExecSelection, id=ID.EXEC_SELECTION)
        tb.AddLabelTool(ID.EXEC_DOCUMENT_FROM_DISK, "Exec_drive",
                        Icon("actions/python-run-drive"),
                        shortHelp="Execute the whole script in the shell from "
                        "the disk. Saves unsaved changes without asking. "
                        "Warning: __file__ and sys.argv currently point to "
                        "eelbrain.__main__ instead of the executed file.")
        self.Bind(wx.EVT_TOOL, self.OnExecDocumentFromDisk,
                  id=ID.EXEC_DOCUMENT_FROM_DISK)

        if self._exec_in_shell_namespace:
            icon = Icon("actions/terminal-on")
        else:
            icon = Icon("actions/terminal-off")
        tb.AddLabelTool(ID.PYDOC_EXEC_ISOLATE, "Exec-Globals", icon,
                        shortHelp="Toggle script execution in the shell's name"
                                  "space (icon terminal on) or in a separate "
                                  "namespace (icon terminal off).")
        self.Bind(wx.EVT_TOOL, self.OnExec_ToggleIsolate, id=ID.PYDOC_EXEC_ISOLATE)

        tb.AddLabelTool(wx.ID_REFRESH, "Update Namespace",
                        Icon("tango/actions/view-refresh"),
                        shortHelp="Refresh local namespace from script (for "
                                  "auto-completion)")
        self.Bind(wx.EVT_TOOL, self.OnUpdateNamespace, id=wx.ID_REFRESH)
        tb.AddSeparator()


        # file operations
        tb.AddLabelTool(wx.ID_NEW, "New", Icon("documents/pydoc-new"),
                        shortHelp="New Document")
        tb.AddLabelTool(wx.ID_OPEN, "Load", Icon("documents/pydoc-open"),
                        shortHelp="Open Document")
        tb.AddLabelTool(wx.ID_SAVEAS, "Save As",
                        Icon("tango/actions/document-save-as"),
                        shortHelp="Save Document as ...")
        tb.AddLabelTool(wx.ID_SAVE, "Save", Icon("tango/actions/document-save"),
                        shortHelp="Save Document")
        tb.AddSeparator()

#        self.Bind(wx.EVT_MENU_CLOSE, self.OnFileClose)
#        self.Bind(wx.EVT_MENU, self.OnClose, id=wx.ID_CLOSE)


        # text search
        self._search_string = None
        search_ctrl = wx.SearchCtrl(tb, id=wx.ID_HELP, style=wx.TE_PROCESS_ENTER,
                                    size=(150, -1))
        search_ctrl.Bind(wx.EVT_TEXT, self.OnSearchMod)
        search_ctrl.Bind(wx.EVT_TEXT_ENTER, self.OnSearchForward)
        search_ctrl.ShowCancelButton(True)
        self.Bind(wx.EVT_SEARCHCTRL_CANCEL_BTN, self.OnSearchCancel, search_ctrl)
        search_ctrl.ShowSearchButton(True)
        self.Bind(wx.EVT_SEARCHCTRL_SEARCH_BTN, self.OnSearchForward, search_ctrl)
#        search_ctrl.SetMenu(self.search_history_menu)
#        self.search_history_menu.Bind(wx.EVT_MENU, self.OnSearchHistory)
        tb.AddControl(search_ctrl)
        self.search_ctrl = search_ctrl
        self.search_str = ''
        self.search_flag = wx.stc.STC_FIND_REGEXP

        tb.AddLabelTool(wx.ID_BACKWARD, "Backward", Icon("tango/actions/go-previous"),
                        shortHelp="Search backwards")
        self.Bind(wx.EVT_TOOL, self.OnSearchBackward, id=wx.ID_BACKWARD)
        tb.AddLabelTool(wx.ID_FORWARD, "Forward", Icon("tango/actions/go-next"),
                        shortHelp="Search forward")
        self.Bind(wx.EVT_TOOL, self.OnSearchForward, id=wx.ID_FORWARD)

        tb.Realize()

    # finalize ---
        if pyfile is True:
            self.bufferOpen()
        elif not pyfile:
            self.bufferCreate()
            if name is not None:
                self.buffer.name = name

        # set icon
        if hasattr(parent, 'eelbrain_icon'):
            self.SetIcon(parent.eelbrain_icon)

        self.SetSize(size)

    def bufferCreate(self, filename=None):
        out = super(PyEditor, self).bufferCreate(filename=filename)
        self.shell.ApplyStyleTo(self.editor.window)

        zoom = self.shell.shell.GetZoom()
        self.editor.window.SetZoom(zoom)

        droptarget.set_for_strings(self.editor.window)
        self.editor.window.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        if filename:
            self.updateTitle()

        return out

    def bufferSave(self):
        """subclassed to catch and display errors

        Returns
        -------
        cancel : bool
            True if the save operation was canceled, False otherwise.
        """
        try:
            cancel = super(PyEditor, self).bufferSave()
        except Exception as exception:
            msg = str(exception) + "\n\n(Press OK and check console for full Exception)"
            dlg = wx.MessageDialog(self,
                                   msg,
                                   "Error saving File",
                                   wx.OK | wx.CENTER | wx.ICON_ERROR)
            dlg.ShowModal()
            raise

        self.updateTitle()
        return cancel

    def bufferSaveAs(self):
        "subclass to catch and display errors"
        try:
            if not hasattr(self.buffer, 'confirmed'):
                # this caused an exception when saving a new document an
                # replacing an existing file
                self.buffer.confirmed = True

            cancel = super(PyEditor, self).bufferSaveAs()
        except Exception as exception:
            msg = str(exception) + "\n\n(Check console for more information)"
            dlg = wx.MessageDialog(self,
                                   msg,
                                   "Error saving File",
                                   wx.OK | wx.CENTER | wx.ICON_ERROR)
            dlg.ShowModal()
            raise

        self.updateTitle()
        return cancel

    def bufferSuggestSave(self):
        """Suggest saving changes.  Return True if user selected Cancel."""
        self.Raise()
        return wx.py.editor.EditorFrame.bufferSuggestSave(self)

    def bufferOpen(self):
        """Subclassed from EditorFrame."""
        cancel = wx.py.editor.EditorFrame.bufferOpen(self)
        if not cancel:
            zoom = self.shell.shell.GetZoom()
            self.editor.window.SetZoom(zoom)
        return cancel

#        cancel = wx.py.editor.EditorFrame.bufferClose(self)
# #        if cancel:
# #            return cancel
#        if self.bufferHasChanged():
#            cancel = self.bufferSuggestSave()
#            if cancel:
#                return cancel
#
#        filedir = ''
#        if self.buffer and self.buffer.doc.filedir:
#            filedir = self.buffer.doc.filedir
#
#        path = ui.ask_file(title="Open Python Script",
#                           message="Select Python Script to load",
#                           ext=[('py', "Python script")],
#                           directory=filedir)
#
#        if isstr(path) and os.path.isfile(path):
#            self.bufferCreate(path)
#        else:
#            logging.info("Py editor OnFileOpen with nonexistent path: %s"%path)
#    def OnClose(self, event):
#        logging.debug('OnClose')
    def _updateStatus(self):
        """
        Update Information in status bar. Replaces from baseclass to add
        position info and change order.
        """
        if self.editor and hasattr(self.editor, 'getStatus'):
            temp = 'Line: %d  |  Column: %d  |  Pos: %d  |  File: %s'
            fpath, line, col = self.editor.getStatus()
            if not self.editor.buffer.doc.filepath:
                fpath = "* unsaved"
            pos = self.editor.window.GetCurrentPos()
            text = temp % (line, col, pos, fpath)
            # end mod
        else:
            text = self._defaultText
        if text != self._statusText:
            self.SetStatusText(text)
            self._statusText = text

    def Destroy(self):
        logging.debug("WxTerm PyEditor.Destroy()")
        self.shell.RemovePyEditor(self)
        super(PyEditor, self).Destroy()

    def Duplicate(self):
        if not self.editor:
            return

        win = self.editor.window
        start_pos, end_pos = win.GetSelection()
        if start_pos == end_pos:  # use current line
            txt, _ = win.GetCurLine()
        else:
            txt = win.GetSelectedText()

        txt = fmtxt.unindent(txt)  # remove leading whitespaces
        self.shell.InsertStrToShell(txt)
        self.shell.Raise()

    def OnActivate(self, event=None):
        # logging.debug(" Shell Activate Event: {0}".format(event.Active))
        if not self.editor:
            return

        if event.Active:
            self.editor.window.SetCaretForeground(wx.Colour(0, 0, 0))
            self.editor.window.SetCaretPeriod(500)
            self.shell.active_editor = self
            if self._exec_in_shell_namespace:
                self.updateNamespace()
        else:
            self.editor.window.SetCaretForeground(wx.Colour(200, 200, 200))
            self.editor.window.SetCaretPeriod(0)

    def CanExec(self):
        return bool(self.editor)

    def Comment(self):
        w = self.editor.window
        start_pos, end_pos = w.GetSelection()
        start = w.LineFromPosition(start_pos)
        end = w.LineFromPosition(end_pos)
        if start > end:
            start, end = end, start
        if start != end and end_pos == w.PositionFromLine(end):
            end -= 1  # if no element in the last line is selected
        lines = range(start, end + 1)
        first = w.GetLine(lines[0])
        if len(first) == 0 or first[0] != '#':  # de-activate
            for i in lines:
                line = w.GetLine(i)
                pos = w.PositionFromLine(i)
                if len(line) > 1 and line[:2] == '##':
                    pass
                elif len(line) > 0 and line[0] == '#':
                    w.InsertText(pos, '#')
                else:
                    w.InsertText(pos, '##')
        else:  # un-comment
            for i in lines:
                while w.GetLine(i)[0] == '#':
                    pos = w.PositionFromLine(i) + 1
                    w.SetSelection(pos, pos)
                    w.DeleteBack()
        start = w.PositionFromLine(start)
        end = w.PositionFromLine(end + 1) - 1
        w.SetSelection(start, end)

    def ExecDocument(self):
        "Execute the whole script in the shell."
        txt = self.editor.getText()

        shell_globals = self._exec_in_shell_namespace
        self.shell.ExecText(txt.encode('utf-8'),
                            title=self.editor.getStatus()[0],
                            comment=None,
                            shell_globals=shell_globals,
                            filepath=self.buffer.doc.filepath,
                            internal_call=True)
        self.updateNamespace()

    def ExecDocumentFromDisk(self):
        "Save unsaved changes and execute file in shell."
        if self.bufferHasChanged():
            if self.bufferSave():
                return  # user canceled
        shell_globals = self._exec_in_shell_namespace
        self.shell.ExecFile(self.buffer.doc.filepath,
                            shell_globals=shell_globals)
        self.updateNamespace()

    def ExecSelection(self):
        """Execute the current selection, or the current line if nothing is
        selected.
        """
        win = self.editor.window
        start_pos, end_pos = win.GetSelection()
        if start_pos == end_pos:  # run current line
            # extended selection including hierarchically subordinate lines
            i_first = win.GetCurrentLine()
            indent = win.GetLineIndentation(i_first)
            i_last = i_first
            n_lines = win.GetLineCount()
            while True:
                if i_last + 1 >= n_lines:
                    break
                elif win.GetLineIndentation(i_last + 1) > indent:
                    pass
                elif not win.GetLine(i_last + 1).strip():
                    pass
                else:
                    start_pos = win.PositionFromLine(i_first)
                    end_pos = win.GetLineEndPosition(i_last)
                    code = win.GetTextRange(start_pos, end_pos)
                    code = fmtxt.unindent(code)
                    try:
                        ast.parse(code, mode='single')
                    except SyntaxError:
                        pass
                    else:
                        break
                i_last += 1

            start_pos = win.PositionFromLine(i_first)
            end_pos = win.GetLineEndPosition(i_last)

            # execute the code
            cmd = win.GetTextRange(start_pos, end_pos)
            if not cmd.strip():
                return
            self.shell.ExecCommand(cmd)

            # move the caret to the next line
            win.SetSelection(end_pos, end_pos)
            self.MoveCaretToNextLine()
        else:  # execute the selection
            txt = self.editor.window.GetSelectedText()
            if not txt:
                return

            # fix selection
            start_pos, end_pos = win.GetSelection()
            if start_pos > end_pos:
                start_pos, end_pos = end_pos, start_pos

            start = win.LineFromPosition(start_pos + 1)
            end = win.LineFromPosition(end_pos)
            if start == end:
                comment = "line %s" % start
            else:
                comment = "lines %s - %s" % (start, end)

            shell_globals = self._exec_in_shell_namespace
            self.shell.ExecText(txt,
                                title=self.editor.getStatus()[0],
                                comment=comment,
                                shell_globals=shell_globals,
                                filepath=self.buffer.doc.filepath,
                                internal_call=True)
        self.updateNamespace()

    def MoveCaretToNextLine(self):
        """Move the caret to the next non-empty line

        Returns
        -------
        pos : int
            New caret position.
        """
        win = self.editor.window
        cur_line = win.GetCurrentLine()
        if cur_line == win.GetLineCount() - 1:
            pos = win.PositionFromLine(cur_line)
        else:
            next_line = cur_line + 1
            while True:
                if next_line == win.GetLineCount() - 1:
                    break
                else:
                    line = win.GetLine(next_line).strip()
                    if line and not line.startswith('#'):
                        break
                    else:
                        next_line += 1

            pos = win.PositionFromLine(next_line)

        win.SetSelection(pos, pos)
        return pos

    def OnExecDocument(self, event=None):
        if self.CanExec():
            self.ExecDocument()

    def OnExecDocumentFromDisk(self, event=None):
        if self.CanExec():
            self.ExecDocumentFromDisk()

    def OnExecSelection(self, event=None):
        "Execute the selection in the shell."
        if self.CanExec():
            self.ExecSelection()

    def OnExec_ToggleIsolate(self, event=None):
        """
        Toggle whether code is executed with the shell's globals or in a
        separate namespace

        """
        self._exec_in_shell_namespace = not self._exec_in_shell_namespace
        if self._exec_in_shell_namespace:
            bmp = Icon("actions/terminal-on")
        else:
            bmp = Icon("actions/terminal-off")
        self.ToolBar.SetToolNormalBitmap(ID.PYDOC_EXEC_ISOLATE, bmp)
        self.updateNamespace()

    def GetCurLine(self):
        return self.editor.window.GetCurLine()

    def OnKeyDown(self, event):
        # these ought to be handles in stc.StyledTextCtrl
        # src/osx_cocoa/stc.py ?
        key = event.GetKeyCode()
        mod = key_mod(event)
        if any(mod):
            mod_str = '-'.join(d for d, s in zip(['ctrl', 'cmd', 'alt'], mod) if s)
            logging.info("Editor OnKeyDown: {0} {1}".format(mod_str, key))

        if mod == [1, 0, 0]:  # [ctrl]
            event.Skip()
        elif mod == [0, 0, 1]:  # alt down
            if key in [315, 317]:  # arrow
                # FIXME: when last line without endline is selected, someting
                #        bad happens
                w = self.editor.window
                w.LineCut()
                if key == 315:  # [up]
                    w.LineUp()
                elif key == 317:  # [down]
                    w.LineDown()
                w.Paste()
                w.LineUp()
                return
            else:
                event.Skip()
#        elif mod == [0, 1, 0]: # command (Os-X)
#            pass
        else:
            event.Skip()

    def InsertLine(self, line):
        w = self.editor.window

        cur_line, cur_line_pos = w.GetCurLine()
        if cur_line_pos == 0:
            pos = w.GetCurrentPos()
            w.InsertText(pos, line)
            newpos = pos + len(line)
            w.SetSelection(newpos, newpos)
            w.NewLine()
        else:
            w.LineEnd()
#            w.NewLine()
            pos = w.GetCurrentPos() + 1
            w.SetSelection(pos, pos)
            w.InsertText(pos, line)

    def OnUpdateNamespace(self, event=None):
        self.updateNamespace()

    # Text Search
    def OnSearchMod(self, event):
        self.search_str = event.GetString()
        self.OnSearchForward(event)

    def OnSearchForward(self, event):
        w = self.editor.window
        old_pos = w.GetSelection()
        new_pos = max(old_pos) + 1
        w.SetSelection(new_pos, new_pos)
        w.SearchAnchor()
        index = w.SearchNext(self.search_flag, self.search_str)
        if index == -1:
            w.SetSelection(min(old_pos), max(old_pos))
        else:
            w.ScrollToLine(w.LineFromPosition(index) - 15)

    def OnSearchBackward(self, event):
        w = self.editor.window
        old_pos = w.GetSelection()
        new_pos = min(old_pos) - 1
        w.SetSelection(new_pos, new_pos)
        w.SearchAnchor()
        index = w.SearchPrev(self.search_flag, self.search_str)
        if index == -1:
            w.SetSelection(min(old_pos), max(old_pos))
        else:
            w.ScrollToLine(w.LineFromPosition(index) - 15)

    def OnSearchCancel(self, event=None):
        self.search_ctrl.Clear()
        self._search_string = None

    def updateNamespace(self):
        if self._exec_in_shell_namespace:
            self.buffer.interp.locals.clear()
            self.buffer.interp.locals.update(self.shell.global_namespace)
        else:
            super(Editor, self).updateNamespace()
#        self.buffer.updateNamespace()

    def updateTitle(self):
        if self.editor is None:
            self.SetTitle("Empty Editor")
        else:
            path = self.editor.getStatus()[0]
            # add to recent paths
            self.shell.recent_item_add(path, 'pydoc')
            # set title
            name = os.path.basename(path)
            self.SetTitle("Script Editor: %s" % name)




'''
Plot :class:`NDVar` objects containing source estimates with mayavi/pysurfer.

.. autosummary::
   :toctree: generated

   activation
   cluster
   dspm
   stat
   surfer_brain

Functions that can be applied to the :class:`surfer.Brain` instances that are
returned by the plotting functions:

.. autosummary::
   :toctree: generated

    bin_table
    copy
    image

'''
# author: Christian Brodbeck
from __future__ import division
from itertools import izip
import os
from tempfile import mkdtemp

import numpy as np

from .._data_obj import asndvar, NDVar, UTS
from ..fmtxt import Image, im_table


__all__ = ['activation', 'dspm', 'surfer_brain', 'stat']


def _idx(i):
    return int(round(i))


def _plot(data, lut, vmin, vmax, *args, **kwargs):
    "Plot depending on source space kind"
    if data.source.kind == 'vol':
        return _voxel_brain(data, lut, vmin, vmax, *args, **kwargs)
    else:
        return surfer_brain(data, lut, vmin, vmax, *args, **kwargs)


def dspm(src, fmin=13, fmax=22, fmid=None, *args, **kwargs):
    """
    Plot a source estimate with coloring for dSPM values (bipolar).

    Parameters
    ----------
    src : NDVar, dims = ([case,] source, [time])
        NDVar with SourceSpace dimension. If stc contains a case dimension,
        the average across cases is taken.
    fmin, fmax : scalar
        Start- and endpoint for the color gradient. Values between -fmin
        and fmin are transparent.
    fmid : None | scalar
        Midpoint for the color gradient. If fmid is None (default) it is set
        half way between fmin and fmax.
    surf : 'inflated' | 'pial' | 'smoothwm' | 'sphere' | 'white'
        Freesurfer surface to use as brain geometry.
    views : str | iterator of str
        View or views to show in the figure.
    colorbar : bool
        Whether to add a colorbar to the figure.
    w, h, axw, axh : scalar
        Layout parameters (figure width/height, subplot width/height).
    background : mayavi color
        Figure background color.
    parallel : bool
        Set views to parallel projection (default ``True``).
    smoothing_steps : None | int
        Number of smoothing steps if data is spatially undersampled (pysurfer
        ``Brain.add_data()`` argument).
    subjects_dir : None | str
        Override the subjects_dir associated with the source space dimension.

    Returns
    -------
    brain : surfer.Brain
        PySurfer Brain instance containing the plot.
    """
    if fmid is None:
        fmid = (fmax + fmin) / 2
    lut = _dspm_lut(fmin, fmid, fmax)
    return _plot(src, lut, -fmax, fmax, *args, **kwargs)


def stat(p_map, param_map=None, p0=0.05, p1=0.01, solid=False, *args,
         **kwargs):
    """
    Plot a statistic in source space.

    Parameters
    ----------
    p_map : NDVar
        Statistic to plot (normally a map of p values).
    param_map : NDVar
        Statistical parameter covering the same data points as p_map. Used
        only for incorporating the directionality of the effect into the plot.
    p0, p1 : scalar
        Threshold p-values for the color map.
    solid : bool
        Use solid color patches between p0 and p1 (default: False - blend
        transparency between p0 and p1).
    surf : 'inflated' | 'pial' | 'smoothwm' | 'sphere' | 'white'
        Freesurfer surface to use as brain geometry.
    views : str | iterator of str
        View or views to show in the figure.
    colorbar : bool
        Whether to add a colorbar to the figure.
    w, h, axw, axh : scalar
        Layout parameters (figure width/height, subplot width/height).
    background : mayavi color
        Figure background color.
    parallel : bool
        Set views to parallel projection (default ``True``).
    smoothing_steps : None | int
        Number of smoothing steps if data is spatially undersampled (pysurfer
        ``Brain.add_data()`` argument).
    subjects_dir : None | str
        Override the subjects_dir associated with the source space dimension.

    Returns
    -------
    brain : surfer.Brain
        PySurfer Brain instance containing the plot.
    """
    pmap, lut, vmax = _p_lut(p_map, param_map, p0=p0, p1=p1, solid=solid)
    return _plot(pmap, lut, -vmax, vmax, *args, **kwargs)


def activation(src, threshold=None, vmax=None, *args, **kwargs):
    """
    Plot activation in source space.

    Parameters
    ----------
    src : NDVar, dims = ([case,] source, [time])
        NDVar with SourceSpace dimension. If stc contains a case dimension,
        the average across cases is taken.
    threshold : scalar | None
        the point at which alpha transparency is 50%. When None,
        threshold = one standard deviation above and below the mean.
    vmax : scalar | None
        the upper range of activation values. values are clipped above this
        range. When None, vmax = two standard deviations above and below the
        mean.
    surf : 'inflated' | 'pial' | 'smoothwm' | 'sphere' | 'white'
        Freesurfer surface to use as brain geometry.
    views : str | iterator of str
        View or views to show in the figure.
    colorbar : bool
        Whether to add a colorbar to the figure.
    w, h, axw, axh : scalar
        Layout parameters (figure width/height, subplot width/height).
    background : mayavi color
        Figure background color.
    parallel : bool
        Set views to parallel projection (default ``True``).
    smoothing_steps : None | int
        Number of smoothing steps if data is spatially undersampled (pysurfer
        ``Brain.add_data()`` argument).
    subjects_dir : None | str
        Override the subjects_dir associated with the source space dimension.

    Returns
    -------
    brain : surfer.Brain
        PySurfer Brain instance containing the plot.
    """
    x = src.mean()
    std = src.std()

    if threshold is None:
        threshold = x + std
    if vmax is None:
        vmax = x + 2 * std
    lut = _activation_lut(threshold, vmax)

    return _plot(src, lut, -vmax, vmax, *args, **kwargs)


def cluster(cluster, vmax=None, *args, **kwargs):
    """Plot a spatio-temporal cluster

    Plots a cluster with the assumption that all non-zero data should be
    visible, while areas that not part of the cluster are 0.

    Parameters
    ----------
    cluster : NDVar
        The cluster.
    vmax : scalar
        Maximum value in the colormap. Default is the maximum value in the
        cluster.
    surf : 'inflated' | 'pial' | 'smoothwm' | 'sphere' | 'white'
        Freesurfer surface to use as brain geometry.
    views : str | iterator of str
        View or views to show in the figure.
    colorbar : bool
        Whether to add a colorbar to the figure.
    w, h, axw, axh : scalar
        Layout parameters (figure width/height, subplot width/height).
    background : mayavi color
        Figure background color.
    parallel : bool
        Set views to parallel projection (default ``True``).
    smoothing_steps : None | int
        Number of smoothing steps if data is spatially undersampled (pysurfer
        ``Brain.add_data()`` argument).
    subjects_dir : None | str
        Override the subjects_dir associated with the source space dimension.

    Returns
    -------
    brain : surfer.Brain
        PySurfer Brain instance containing the plot.
    """
    if vmax is None:
        vmax = max(cluster.x.max(), -cluster.x.min())

    lut = _dspm_lut(0, vmax / 10, vmax)
    return _plot(cluster, lut, -vmax, vmax, *args, **kwargs)


def surfer_brain(src, colormap='hot', vmin=0, vmax=9, surf='smoothwm',
                 views=['lat', 'med'], colorbar=True, time_label='%.3g s',
                 w=None, h=None, axw=None, axh=None, background=None,
                 parallel=True, smoothing_steps=None, subjects_dir=None):
    """Create a PySurfer Brain object with a data layer

    Parameters
    ----------
    src : NDVar, dims = ([case,] source, [time])
        NDVar with SourceSpace dimension. If stc contains a case dimension,
        the average across cases is taken.
    colormap :
        Colormap for PySurfer.
    vmin, vmax : scalar
        Endpoints for the colormap.
    surf : 'inflated' | 'pial' | 'smoothwm' | 'sphere' | 'white'
        Freesurfer surface to use as brain geometry.
    views : str | iterator of str
        View or views to show in the figure.
    colorbar : bool
        Whether to add a colorbar to the figure.
    w, h, axw, axh : scalar
        Layout parameters (figure width/height, subplot width/height).
    background : mayavi color
        Figure background color.
    parallel : bool
        Set views to parallel projection (default ``True``).
    smoothing_steps : None | int
        Number of smoothing steps if data is spatially undersampled (pysurfer
        ``Brain.add_data()`` argument).
    subjects_dir : None | str
        Override the subjects_dir associated with the source space dimension.

    Returns
    -------
    brain : surfer.Brain
        PySurfer Brain instance containing the plot.
    """
    from surfer import Brain

    if isinstance(views, basestring):
        views = [views]
    elif not isinstance(views, list):
        views = list(views)

    src = asndvar(src)  # , sub=None, ds=ds)
    if src.has_case:
        src = src.summary()

    if src.source.lh_n and src.source.rh_n:
        hemi = 'split'
        n_hemi = 2
    elif src.source.lh_n:
        hemi = 'lh'
        n_hemi = 1
    elif not src.source.rh_n:
        raise ValueError('No data')
    else:
        hemi = 'rh'
        n_hemi = 1

    title = None
    config_opts = {}
    if w is not None:
        config_opts['width'] = w
    elif axw is not None:
        config_opts['width'] = axw * n_hemi
    else:
        config_opts['width'] = 500 * n_hemi

    if h is not None:
        config_opts['height'] = h
    elif axh is not None:
        config_opts['height'] = axh * len(views)
    else:
        config_opts['height'] = 400 * len(views)

    if background is not None:
        config_opts['background'] = background

    if subjects_dir is None:
        subjects_dir = src.source.subjects_dir

    brain = Brain(src.source.subject, hemi, surf, True, title,
                  config_opts=config_opts, views=views,
                  subjects_dir=subjects_dir)

    # general PySurfer data args
    alpha = 1
    if smoothing_steps is None and src.source.kind == 'ico':
        smoothing_steps = src.source.grade + 1

    if src.has_dim('time'):
        times = src.time.times
        data_dims = ('source', 'time')
    else:
        times = None
        data_dims = ('source',)

    if src.source.lh_n:
        if hemi == 'lh':
            colorbar_ = colorbar
            colorbar = False
            time_label_ = time_label
            time_label = None
        else:
            colorbar_ = False
            time_label_ = None

        src_hemi = src.sub(source='lh')
        data = src_hemi.get_data(data_dims)
        vertices = src.source.lh_vertno
        brain.add_data(data, vmin, vmax, None, colormap, alpha, vertices,
                       smoothing_steps, times, time_label_, colorbar_, 'lh')

    if src.source.rh_n:
        src_hemi = src.sub(source='rh')
        data = src_hemi.get_data(data_dims)
        vertices = src.source.rh_vertno
        brain.add_data(data, vmin, vmax, None, colormap, alpha, vertices,
                       smoothing_steps, times, time_label, colorbar, 'rh')

    if parallel:  # set parallel view
        if surf == 'inflated':
            camera_scale = 95
        else:
            camera_scale = 65

        for figs in brain._figures:
            for fig in figs:
                fig.scene.camera.parallel_scale = camera_scale
                fig.scene.camera.parallel_projection = True

    return brain


def _dspm_lut(fmin, fmid, fmax):
    """Create a color look up table (lut) for a dSPM plot

    Parameters
    ----------
    fmin, fmid, fmax : scalar
        Start-, mid- and endpoint for the color gradient.
    """
    if not (fmin < fmid < fmax):
        raise ValueError("Invalid colormap, we need fmin < fmid < fmax")

    n = 256
    lut = np.zeros((n, 4), dtype=np.uint8)
    i0 = _idx(n / 2)
    imin = _idx((fmin / fmax) * i0)
    min_n = i0 - imin
    min_p = i0 + imin
    imid = _idx((fmid / fmax) * i0)
    mid_n = i0 - imid
    mid_p = i0 + imid

    # red end
    lut[i0:, 0] = 255
    lut[mid_p:, 1] = np.linspace(0, 255, n - mid_p)

    # blue end
    lut[:i0, 2] = 255
    lut[:mid_n, 0] = np.linspace(127, 0, mid_n)
    lut[:mid_n, 1] = np.linspace(127, 0, mid_n)

    # alpha
    lut[:mid_n, 3] = 255
    lut[mid_n:min_n, 3] = np.linspace(255, 0, min_n - mid_n)
    lut[min_n:min_p, 3] = 0
    lut[min_p:mid_p, 3] = np.linspace(0, 255, mid_p - min_p)
    lut[mid_p:, 3] = 255

    return lut


def _p_lut(pmap, tmap, p0=0.05, p1=0.01, n=256, solid=False):
    """Creat a color look up table (lut) for p-values

    Parameters
    ----------
    pmap : NDVar
        Map of p-values.
    tmap : NDVar
        Map of signed statistic (only used to code the sign of each p-value).
    p0 : scalar
        Highest p-vale that should be visible.
    p1 : scalar
        P-value where the colormap changes from ramping alpha to ramping color.
    n : int
        Number of color categories in the lut.
    solid : bool
        Instead of ramping alpha/color hue, create steps at p0 and p1.
    """
    if p1 >= p0:
        raise ValueError("p1 needs to be smaller than p0.")

    pstep = 2 * p0 / _idx(n / 2 - 1)

    # max p-value that needs to be represented
    vmax = p0 + pstep

    # bring interesting p-values to the range [pstep vmax]
    pmap = vmax - pmap

    # set uninteresting values to zero
    pmap.x.clip(0, vmax, pmap.x)

    # add sign to p-values
    if tmap is not None:
        pmap.x *= np.sign(tmap.x)

    # http://docs.enthought.com/mayavi/mayavi/auto/example_custom_colormap.html
    lut = np.zeros((n, 4), dtype=np.uint8)

    middle = n / 2
    p0n = _idx(p0 / pstep)
    p0p = n - p0n
    p1n = _idx(p1 / pstep)
    p1p = n - p1n

    # negative colors
    lut[:middle, 2] = 255
    lut[:p1n, 0] = np.linspace(255, 0, p1n)

    # positive colors
    lut[middle:, 0] = 255
    lut[p1p:, 1] = np.linspace(0, 255, n - p1p)

    # alpha
    if solid:
        lut[:p0n, 3] = 255
        lut[p0p:, 3] = 255
    else:
        lut[:p1n, 3] = 255
        lut[p1n:p0n, 3] = np.linspace(255, 0, p0n - p1n)
        lut[p0p:p1p, 3] = np.linspace(0, 255, p1p - p0p)
        lut[p1p:, 3] = 255

    return pmap, lut, vmax



def _activation_lut(threshold=3, vmax=8):
    """
    Creates a lookup table containing a color map for plotting stc activation.

    Parameters
    ----------
    threshold : scalar
        threshold is point at which values gain 50% visibility.
        from threshold and beyond, alpha = 100% visibility.
    vmax : scalar
        the upper range of activation values. values are clipped above this range. When None,
        vmax = two standard deviations above and below the mean.

    Notes
    -----
    Colors:
    - negative is blue.
    - super negative is purple.
    - positive is red.
    - super positive is yellow.
    """

    values = np.linspace(-vmax, vmax, 256)
    trans_uidx = np.argmin(np.abs(values - threshold))
    trans_lidx = np.argmin(np.abs(values + threshold))

    # Transparent ramping
    lut = np.zeros((256, 4), dtype=np.uint8)
    lut[127:trans_uidx, 3] = np.linspace(0, 128, trans_uidx - 127)
    lut[trans_lidx:127, 3] = np.linspace(128, 0, 127 - trans_lidx)
    lut[trans_uidx:, 3] = 255
    lut[:trans_lidx, 3] = 255

    # negative -> Blue
    lut[:127, 2] = 255
    # super negative -> Purple
    lut[:trans_lidx, 0] = np.linspace(0, 255, trans_lidx)
    # positive -> Red
    lut[127:, 0] = 255
    # super positive -> Yellow
    lut[trans_uidx:, 1] = np.linspace(0, 255, 256 - trans_uidx)

    return lut


def _voxel_brain(data, lut, vmin, vmax):
    """Plot spheres for volume source space

    Parameters
    ----------
    data : NDVar
        Data to plot.
    lut : array
        Color LUT.
    vmin, vmax : scalar
        Data range.
    """
    if data.dimnames != ('source',):
        raise ValueError("Can only plot 1 dimensional source space NDVars")

    from mayavi import mlab

    x, y, z = data.source.coordinates.T

    figure = mlab.figure()
    mlab.points3d(x, y, z, scale_factor=0.002, opacity=0.5)
    pts = mlab.points3d(x, y, z, data.x, vmin=vmin, vmax=vmax)
    pts.module_manager.scalar_lut_manager.lut.table = lut
    return figure


def bin_table(ndvar, tstart=None, tstop=None, tstep=0.1, surf='smoothwm',
              views=['lat', 'med'], summary=np.sum):
    """Create a table with images for time bins

    Parameters
    ----------
    ndvar : NDVar (time x source)
        Data to be plotted.
    tstart : None | scalar
        Time point of the start of the first bin (inclusive; None to use the
        first time point in ndvar).
    tstop : None | scalar
        End of the last bin (exclusive; None to end with the last time point
        in ndvar).
    tstep : scalar
        Size of each bin (in seconds).
    surf : 'inflated' | 'pial' | 'smoothwm' | 'sphere' | 'white'
        Freesurfer surface to use as brain geometry.
    views : list of str
        Views to display (for each hemisphere, lh first).
    summary : callable
        How to summarize data in each time bin. The value should be a function
        that takes an axis parameter (e.g., numpy summary functions like
        numpy.sum, numpy.mean, numpy.max, ..., default is numpy.sum).

    Returns
    -------
    images : Image
        FMTXT Image object that can be saved as SVG or integrated into an
        FMTXT document.
    """
    data = _bin_data(ndvar, tstart, tstop, tstep, summary)
    ims = []
    vmax = max(abs(data.min()), data.max())

    hemis = []
    if data.source.lh_n:
        hemis.append('lh')
    if data.source.rh_n:
        hemis.append('rh')

    for hemi in hemis:
        hemi_data = data.sub(source=hemi)
        brain = cluster(hemi_data, vmax, surf, views[0], colorbar=False, w=300,
                        h=250, time_label=None)
        im = brain.screenshot_single('rgba', True)

        hemi_lines = [[] for _ in views]
        for i in xrange(len(data.time)):
            brain.set_data_time_index(i)
            for line, view in izip(hemi_lines, views):
                brain.show_view(view)
                im = brain.screenshot_single('rgba', True)
                line.append(im)
        ims += hemi_lines
#         brain.close() # causes segfault in wx

    bins = data.info['bins']
    header = ['%i - %i ms' % (t0 * 1000, t1 * 1000) for t0, t1 in bins]
    im = im_table(ims, header)

    return im


def _bin_data(ndvar, tstart, tstop, tstep, summary):
    data = ndvar.get_data(('source', 'time'))

    # times
    if tstart is None:
        tstart = ndvar.time.tmin
    if tstop is None:
        tstop = ndvar.time.tmax + ndvar.time.tstep
    times = np.arange(tstart, tstop, tstep)
    if times[-1] < tstop:
        times = np.append(times, tstop)

    n_bins = len(times) - 1
    x = np.empty((len(data), n_bins))
    bins = []
    for i in xrange(n_bins):
        t0 = times[i]
        t1 = times[i + 1]
        bins.append((t0, t1))
        idx = ndvar.time.dimindex((t0, t1))
        x[:, i] = summary(data[:, idx], axis=1)

    time = UTS(tstart + tstep / 2, tstep, n_bins)
    dims = (ndvar.source, time)
    info = ndvar.info.copy()
    info['bins'] = bins
    out = NDVar(x, dims, info, ndvar.name)
    return out


def connectivity(source):
    """Plot source space connectivity

    Parameters
    ----------
    source : SourceSpace | NDVar
        Source space or NDVar containing the source space.

    Returns
    -------
    figure : mayavi Figure
        The figure.
    """
    from mayavi import mlab

    if isinstance(source, NDVar):
        source = source.get_dim('source')

    connections = source.connectivity()
    coords = source.coordinates
    x, y, z = coords.T

    figure = mlab.figure()
    src = mlab.pipeline.scalar_scatter(x, y, z, figure=figure)
    src.mlab_source.dataset.lines = connections
    lines = mlab.pipeline.stripper(src)
    mlab.pipeline.surface(lines, colormap='Accent', line_width=1, opacity=1.,
                          figure=figure)
    return figure


def image(brain, filename, alt=None, close=False):
    """Create an FMText Image from a brain instance

    Parameters
    ----------
    brain : Brain
        Pysurfer Brain instance.
    filename : str
        Filename for the image (should end with the desired extension).
    alt : None | str
        Alternate text, placeholder in case the image can not be found
        (HTML `alt` tag).
    close : bool
        Close the brain window after creating the image.
    """
    im = brain.screenshot('rgba', True)
    if close:
        brain.close()
    return Image.from_array(im, filename, alt)


def copy(brain):
    "Copy the figure to the clip board"
    import wx

    tempdir = mkdtemp()
    tempfile = os.path.join(tempdir, "brain.png")
    brain.save_image(tempfile)

    bitmap = wx.Bitmap(tempfile, wx.BITMAP_TYPE_PNG)
    bitmap_obj = wx.BitmapDataObject(bitmap)

    if not wx.TheClipboard.IsOpened():
        open_success = wx.TheClipboard.Open()
        if open_success:
            wx.TheClipboard.SetData(bitmap_obj)
            wx.TheClipboard.Close()
            wx.TheClipboard.Flush()



"""
the main application for Eelbrain's wx-terminal

"""

import logging
import os
import sys

import wx

from .shell import ShellFrame


class ModalDummyDialog(wx.Dialog):
    """Modal Dialog to show in wxPython 2.9.x on startup which makes invisible
    windows visible.
    """
    def __init__(self, message, title):
        wx.Dialog.__init__(self, None, -1, title, size=(200, 200))
        self.CenterOnScreen(wx.BOTH)

    def CloseAndDestroy(self):
        self.Close()
        self.Destroy()


class MainApp(wx.App):
    """
    The main application (:py:class:`wx.App` subclass). Creates the shell
    instance.

    """
    def __init__(self, py2app):
        self.py2app = py2app
        wx.App.__init__(self)  # , redirect=redirect, filename=filename)

    def OnInit(self):
        self.shell = ShellFrame(app=self)
        self.SetTopWindow(self.shell)
        self.shell.Show()
        if self.py2app and wx.__version__ >= '2.9':
            dlg = ModalDummyDialog('dsa', 'dsa')
            wx.CallLater(200, dlg.CloseAndDestroy)
            dlg.ShowModal()

        if len(sys.argv) > 1:
            for arg in sys.argv[1:]:
                self.shell.OnFileOpen(None, path=arg)

        return True

    def BringWindowToFront(self):
        win = self.shell.get_active_window()
        if win is None:
            self.shell.Show()
            self.shell.Raise()
        else:
            win.Raise()

    def MacOpenFile(self, fname):
        if not fname.endswith('bin/eelbrain'):
            self.MacOpenFiles([fname])

    def MacOpenFiles(self, filenames):
        logging.debug("MacOpenFiles: %s" % repr(filenames))
        for filename in filenames:
            if os.path.isfile(filename):
                self.shell.OnFileOpen(path=filename)

    def MacReopenApp(self):
        """Called when the doc icon is clicked"""
        self.BringWindowToFront()

    def OnActivate(self, event):
        # if this is an activate event, rather than something else, like iconize.
        if event.GetActive():
            self.BringWindowToFront()
        event.Skip()



"""Plotting for data-objects

Plotting multidimensional data (:class:`NDVar` objects)
-------------------------------------------------------

.. automodule:: eelbrain.plot.uts

.. autosummary::
   :toctree: generated

   UTSClusters
   UTSStat
   UTS


.. automodule:: eelbrain.plot.utsnd

.. autosummary::
   :toctree: generated

   Array
   Butterfly


.. automodule:: eelbrain.plot.topo

.. autosummary::
   :toctree: generated

   TopoArray
   TopoButterfly
   Topomap


.. automodule:: eelbrain.plot.sensors

.. autosummary::
   :toctree: generated

   SensorMap2d
   SensorMaps


Submodules
----------

.. currentmodule:: eelbrain

.. autosummary::
   :toctree: generated

    plot.brain
    plot.uv


.. _plotting-general:

General Parameters
------------------

Xax
^^^

Many plots have an ``Xax`` parameter. This parameter takes a categorial data
object. The data of the plotted variable will be split into the catories in
``Xax``, and for every cell in ``Xax`` a separate subplot will be plotted.

For example, while

    >>> plot.Butterfly('meg', ds=ds)

will create a single Butterfly plot of the average response,

    >>> plot.Butterfly('meg', 'subject', ds=ds)

where ``'subject'`` is the ``Xax`` parameter, will create a separate subplot
for every subject with its average response.


Layout
^^^^^^

Most plots that also share certain layout keyword arguments. By default, all
those parameters are determined automatically, but individual values can be
specified manually by supplying them as keyword arguments.

h, w : scalar
    Height and width of the figure.
axh, axw : scalar
    Height and width of the axes.
nrow, ncol : None | int
    Limit number of rows/columns. If neither is specified, a square layout
    is produced
ax_aspect : scalar
    Width / height aspect of the axes.

Plots that do take those parameters can be identified by the ``**layout`` in
their function signature.

"""
from sensors import SensorMaps, SensorMap2d
from topo import TopoArray, TopoButterfly, Topomap
from uts import UTSStat, UTS, UTSClusters
from utsnd import Array, Butterfly
from . import brain, uv

from ._base import configure_backend



'''
Created on Sep 13, 2013

@author: Christian M Brodbeck
'''
import random

import numpy as np

from .._data_obj import isvar, isndvar


def _resample_params(N, samples):
    """Decide whether to do permutations or random resampling

    Parameters
    ----------
    N : int
        Number of observations.
    samples : int
        ``samples`` parameter (number of resampling iterations, or < 0 to
        sample all permutations).

    Returns
    -------
    n_samples : int
        Adapted number of resamplings that will be done.
    samples : int
        Samples parameter for the resample function (-1 to do all permutations,
        otherwise same as n_samples).
    """
    n_perm = 2 ** N
    if n_perm - 1 <= samples:
        samples = -1

    if samples < 0:
        n_samples = n_perm - 1
    else:
        n_samples = samples

    return n_samples, samples


def resample(Y, samples=10000, replacement=False, unit=None, sign_flip=False,
             seed=0):
    """
    Generator function to resample a dependent variable (Y) multiple times

    Parameters
    ----------
    Y : Var | NDVar
        Variable which is to be resampled.
    samples : int
        Number of samples to yield. If < 0, all possible permutations are
        performed.
    replacement : bool
        whether random samples should be drawn with replacement or without.
    unit : categorial
        Factor specifying unit of measurement (e.g. subject). If unit is
        specified, resampling proceeds by first resampling the categories of
        unit (with or without replacement) and then shuffling the values
        within units (no replacement).
    sign_flip : bool
        Instead of shuffling the observations in the data (default), randomly
        do or do not flip the sign of each observation. If the number of
        possible permutations is smaller than ``samples``, ``resample(...,
        sign_flip=True)`` iterates over all possible permutations.
    seed : None | int
        Seed the random state of the relevant randomization module
        (:mod:`random` or :mod:`numpy.random`) to make replication possible.
        None to skip seeding (default 0).

    Returns
    -------
    Iterator over Y_resampled. The same copy of ``Y`` is yielded in each
    iteration with different data.
    """
    samples = int(samples)
    if isvar(Y):
        pass
    elif isndvar(Y):
        if not Y.has_case:
            raise ValueError("Need NDVar with cases")
    else:
        raise TypeError("Need Var or NDVar")

    Yout = Y.copy('{name}_resampled')

    if samples < 0 and not sign_flip:
        err = "Complete permutation for resampling through reordering"
        raise NotImplementedError(err)

    if seed is not None:
        if sign_flip:
            random.seed(seed)
        else:
            np.random.seed(seed)

    if sign_flip:
        if replacement is not False:
            err = ("`replacement` parameter does not apply when sign_flip is "
                   "True")
            raise ValueError(err)
        elif unit is not None:
            err = "`unit` parameter does not apply when sign_flip is True"
            raise ValueError(err)

        # determine possible number of permutations
        N = len(Y)
        n_perm = 2 ** N
        if samples < 0:
            # do all permutations
            sample_sequences = xrange(1, n_perm)
        else:
            # random resampling
            sample_sequences = random.sample(xrange(1, n_perm), samples)

        sign_shape = (N,) + (1,) * (Y.ndim - 1)
        mult = 2 ** np.arange(N, dtype=np.uint32).reshape(sign_shape)
        buffer_ = np.empty(sign_shape, dtype=np.uint32)
        for i in sample_sequences:
            np.floor_divide(i, mult, buffer_, dtype=np.uint32)
            buffer_ %= 2
            sign = np.where(buffer_, -1, 1)
            np.multiply(Y.x, sign, Yout.x)
            yield Yout
    elif unit is None:
        if replacement:
            N = len(Y)
            for _ in xrange(samples):
                index = np.random.randint(N, N)
                Yout.x = Y.x[index]
                yield Yout
        else:  # OK
            for _ in xrange(samples):
                np.random.shuffle(Yout.x)
                yield Yout
    else:
        if replacement:
            raise NotImplementedError("Replacement and units")
        else:
            idx_orig = np.arange(len(Y))
            idx_perm = np.arange(len(Y))
            unit_idxs = [np.nonzero(unit == cell)[0] for cell in unit.cells]
            for _ in xrange(samples):
                for idx_ in unit_idxs:
                    v = idx_orig[idx_]
                    np.random.shuffle(v)
                    idx_perm[idx_] = v
                Yout.x[idx_perm] = Y.x
                yield Yout



'''
Created on Nov 8, 2011

@author: christian
'''
import logging
import os

import wx
from wx.py.frame import ID_SAVEHISTORY, ID_AUTO_SAVESETTINGS

from .._wxutils import ID


class PreferencesDialog(wx.Frame):
    def __init__(self, shell, Id=wx.ID_ANY, pos=wx.DefaultPosition,
                 size=(500, 50), style=wx.DEFAULT_FRAME_STYLE):  # wx.DefaultSize
        """
        tutorial on layout:
        http://zetcode.com/wxpython/layout/

        """
        title = "Eelbrain Preferences"
        wx.Frame.__init__(self, shell, Id, title, pos, size, style)
        self.config = shell.config

        pref_sizer = wx.BoxSizer(wx.VERTICAL)
        border = 3
        ALIGN_LEFT = wx.ALIGN_LEFT | wx.ALL
        ALIGN_RIGHT = wx.ALIGN_RIGHT | wx.ALL

    # Startup Script ---
        panel = wx.Panel(self, -1)  # , size=(500,300))
#         panel.SetBackgroundColour("BLUE")
        sizer = wx.BoxSizer(wx.HORIZONTAL)

        txt = wx.StaticText(panel, label="Startup Script:")
        sizer.Add(txt, 0, ALIGN_LEFT, border)

        chk = wx.CheckBox(panel, ID.ENABLE_STARTUP_SCRIPT, "Enable")
        state = self.config.ReadBool('Options/ExecStartupScript', True)
        chk.SetValue(state)
        chk.Bind(wx.EVT_CHECKBOX, self.OnConfigCheckbox)
        sizer.Add(chk, 0, ALIGN_LEFT, border)

        btn = wx.Button(panel, label="Show File")
        btn.Bind(wx.EVT_BUTTON, self.OnShowStartupScript)
        sizer.Add(btn, 0, ALIGN_LEFT, border)

        btn = wx.Button(panel, label="Edit")
        btn.Bind(wx.EVT_BUTTON, self.OnEditStartupScript)
        sizer.Add(btn, 0, ALIGN_LEFT, border)

        btn = wx.Button(panel, wx.ID_HELP)
        btn.Bind(wx.EVT_BUTTON, self.OnHelpStartupScript)
        sizer.Add(btn, 0, ALIGN_RIGHT, border)

        pref_sizer.Add(sizer, 0, border)

    # Autosave ---
        sizer = wx.BoxSizer(wx.HORIZONTAL)

        txt = wx.StaticText(panel, label='Autosave:')
        sizer.Add(txt, 0, ALIGN_LEFT, border)

        chk = wx.CheckBox(panel, ID_SAVEHISTORY, "History")
        state = self.config.ReadBool('Options/AutoSaveHistory', True)
        chk.SetValue(state)
        chk.Bind(wx.EVT_CHECKBOX, shell.OnSaveHistory)
        self.checkboxSaveHistory = chk
        sizer.Add(chk, 0, ALIGN_LEFT, border)

        chk = wx.CheckBox(panel, ID_AUTO_SAVESETTINGS, "Settings")
        state = self.config.ReadBool('Options/AutoSaveSettings', True)
        chk.SetValue(state)
        chk.Bind(wx.EVT_CHECKBOX, shell.OnAutoSaveSettings)
        self.checkboxSaveSettings = chk
        sizer.Add(chk, 0, ALIGN_LEFT, border)

        pref_sizer.Add(sizer, 0)

    # Font ---
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        button = wx.Button(panel, ID.SET_FONT, "Font")
        self.Bind(wx.EVT_BUTTON, self.OnSetFont, id=ID.SET_FONT)
        sizer.Add(button, 0, ALIGN_LEFT, border)
        pref_sizer.Add(sizer, 0)

        panel.SetSizer(pref_sizer)

        pref_sizer.Fit(self)

    def OnConfigCheckbox(self, event):
        id_ = event.GetId()
        state = event.IsChecked()
        if id_ == ID.ENABLE_STARTUP_SCRIPT:
            self.config.WriteBool('Options/ExecStartupScript', state)
            self.Parent.execStartupScript = state
        self.config.Flush()

    def OnHelpStartupScript(self, event):
        msg = ("The startup script is executed every time the Eelbrain "
               "application is opened. If the PYTHONSTARTUP environment "
               "variable is defined, the file designated by the path stored "
               "in this variable is used instead of Eelbrain's startup "
               "script.")
        dlg = wx.MessageDialog(self, msg, "Help: Startup Script",
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def OnEditStartupScript(self, event):
        path = self.Parent.startupScript
        if not os.path.exists(path):
            with open(path, 'w') as fid:
                fid.write("# Eelbrain startup script\n")
        self.Parent.create_py_editor(pyfile=path)

    def OnSetFont(self, event):
        try:
            cur_size = int(self.config.Read('font size'))
        except:
            cur_size = 12

        font = wx.Font(pointSize=cur_size,
                       family=wx.FONTFAMILY_UNKNOWN,
                       style=wx.FONTSTYLE_NORMAL,
                       weight=wx.FONTWEIGHT_NORMAL,
                       face=self.config.Read('font'))
        data = wx.FontData()
        data.EnableEffects(True)
        data.SetInitialFont(font)
        data.SetColour(self.config.Read('font color'))
        dlg = wx.FontDialog(self, data)
        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetFontData()
            font_ = data.GetChosenFont()
            font = font_.GetFaceName()
            size = font_.GetPointSize()
            color_ = data.GetColour()
            color = color_.GetAsString(wx.C2S_HTML_SYNTAX)

            logging.debug('You selected: "%s", %d points, color %s\n' %
                          (font, size, color))

            self.config.Write("font", font)
            self.config.Write("font size", str(size))
            self.config.Write("font color", color)
            self.config.Flush()
            self.Parent.ApplyStyle()

        dlg.Destroy()

    def OnShowStartupScript(self, event):
        path = os.path.dirname(self.Parent.startupScript)
        os.system('open %s' % path)



"""
statistical tests for data objects

"""
from __future__ import division

import itertools

import numpy as np
import scipy.stats
from matplotlib import pyplot as P

from .. import fmtxt
from .._data_obj import (ascategorial, asfactor, assub, asvar, cellname,
                         Celltable, Factor, isvar)
from .permutation import resample


__test__ = False


def lilliefors(data, formatted=False, **kwargs):
    """Lilliefors test for normal distribution.

    The Lilliefors test is an adaptation of the Kolmogorov-Smirnov test. It
    is used to test the null hypothesis that data come from a normally
    distributed population, when the null hypothesis does not specify which
    normal distribution, i.e. does not specify the expected value and variance.

    Parameters
    ----------
    data : array_like

    formatted : bool
        Return a single string with the results instead of the numbers.
    kwargs :
        All keyword arguments are forwarded to :func:`scipy.stats.kstest`.

    Returns
    -------
    D : float
        The D-value of the Kolmogorov-Smirnov Test
    p_estimate : float
        The approximate p value according to Dallal and Wilkinson (1986).
        Requires minimal sample size of 5. p is reasonably accurate only when
        it is <= .1 (cf. Dallal and Wilkens).

    Notes
    -----
    Uses the scipy.stats.kstest implementation of the Kolmogorov-Smirnov test.

    References
    ----------
    Dallal, G. E. and Wilkinson, L. (1986). An Analytic Approximation to the
            Distribution of Lilliefors's Test Statistic for Normality. The
            American Statistician, 40(4), 294--296.
    Lilliefors, H. W. (1967). On the Kolmogorov-Smirnov Test for Normality
            with Mean and Variance Unknown. Journal of the American
            Statistical Association, 62(318), 399--402.
    """
    # p values agree with R lillie.test (nortest package) on low values of p.
    # lillie.test adjusts something at p>.1
    # http://pbil.univ-lyon1.fr/library/nortest/R/nortest
    data = np.asarray(data)
    N = len(data)  # data.shape[-1] #axis]
    assert N >= 5, "sample size must be greater than 4"
    # perform Kolmogorov-Smirnov with estimated mean and std
    m = np.mean(data)  # , axis=axis)
    s = np.std(data, ddof=1)  # , axis=axis)
    D, ks_p = scipy.stats.kstest(data, 'norm', args=(m, s), **kwargs)
    # approximate p (Dallal)
    if N > 100:
        D *= (N / 100) ** .49
        N = 100
    p_estimate = np.exp(-7.01256 * D ** 2 * (N + 2.78019)             \
                     + 2.99587 * D * (N + 2.78019) ** .5 - .122119    \
                     + .974598 / (N ** .5) + 1.67997 / N)
    # approximate P (Molin & Abdi)
    L = D  # ???
    b2 = 0.08861783849346
    b1 = 1.30748185078790
    b0 = 0.37872256037043
    A = (-(b1 + N) + np.sqrt((b1 + N) ** 2 - 4 * b2 * (b0 - L ** -2))) / 2 * b2
    Pr = -.37782822932809         \
         + 1.67819837908004 * A     \
         - 3.02959249450445 * A ** 2  \
         + 2.80015798142101 * A ** 3  \
         - 1.39874347510845 * A ** 4  \
         + 0.40466213484419 * A ** 5  \
         - 0.06353440854207 * A ** 6  \
         + 0.00287462087623 * A ** 7  \
         + 0.00069650013110 * A ** 8  \
         - 0.00011872227037 * A ** 9  \
         + 0.00000575586834 * A ** 10
    #
    if formatted:
        txt = "D={0:.4f}, Dallal p={1:.4f}, Molin&Abdi p={2:.4f}"
        return txt.format(D, p_estimate, Pr)
    else:
        return D, p_estimate



def _hochberg_threshold(N, alpha=.05):
    j = np.arange(N)
    threshold = alpha / (N - j)
    return threshold



def mcp_adjust(ps, method='Hochberg'):
    """Adjust p-values for multiple comparison

    Parameters
    ----------
    ps : sequence of scalar
        P-values.
    method : 'hochberg' | 'bonferroni' | 'holm'
        Correction method. Default is 'hochberg'.

    Returns
    -------
    adjusted_ps : list of scalar
        Adjusted p-values.
    """
    n = len(ps)
    method_ = method.lower()
    if method_ == 'bonferroni':
        ps_adjusted = [p * n for p in ps]
    elif method_ in ('hochberg', 'holm'):
        ascsort = np.argsort(ps)
        ps_asc = np.array(ps)[ascsort]
        iout_asc = np.arange(n)[ascsort]
        ps_adjusted = [-1] * n
        p_buffer = 1
        if method_ == 'holm':
            for i in range(n):
                p = ps_asc[i]
                p_adj = (n - i) * p
                p_buffer = max(p_buffer, p_adj)
                ps_adjusted[iout_asc[i]] = p_buffer
        elif method_ == 'hochberg':
            for i in range(1, n + 1):
                p = ps_asc[-i]
                p_adj = (i) * p
                p_buffer = min(p_adj, p_buffer)
                ps_adjusted[iout_asc[-i]] = p_buffer
    else:
        msg = ('%r is not a valid argument for multiple comparison correction '
               'method' % method)
        raise ValueError(msg)

    return ps_adjusted



def _get_correction_caption(corr, n):
    if corr == 'Hochberg':
        return "(* Corrected after Hochberg, 1988)"
    elif corr == 'Bonferroni':
        return "(* Bonferroni corrected for %i tests)" % n
    elif corr == 'Holm':
        return "(* Corrected after Holm)"
    else:
        return "(* Uncorrected)"



def star(p_list, out=str, levels=True, trend=False, corr='Hochberg',
         eq_strlen=False):
    """

    out=str: convert n stars into string containing '**'
       =int: leave n stars as integer

    corr: "Bonferroni"
          "Hochberg"

    levels: {p: string, ...} dictionary. Default (levels=True) creates the
            levels {.05 : '*',    . trend=True adds {.1: "'"}.
                    .01 : '**',
                    .001: '***'}

    eq_strlen: ("equalize string lengths") when strings are returned make sure
               they all have the same length (False by default).

    """
    # set default levels
    if levels is True:
        levels = {.05 : '*',
                  .01 : '**',
                  .001: '***'}
        if trend is True:
            levels[.1] = "`"  # "`"
        elif fmtxt.isstr(trend):
            levels[.1] = trend
    elif trend:
        raise AssertionError("'trend' kwarg only meaningful when levels is True")

    a_levels = sorted(levels.keys(), reverse=True)
    symbols = [''] + [levels[p] for p in a_levels]

    # allow input (p_list) to contain single p-value
    if np.iterable(p_list):
        int_out = False
    else:
        int_out = True
        p_list = [p_list]

    N = len(p_list)
    nstars = np.zeros(N, dtype=int)
    if corr:
        p_list = mcp_adjust(p_list, corr)
    p_list = np.asarray(p_list)
    for a in a_levels:
        nstars += (p_list <= a)

    # out
    if out == str:
        if eq_strlen:
            maxlen = max([len(s) for s in symbols])
            out = [(symbols[n]).ljust(maxlen) for n in nstars]
        else:
            out = [symbols[n] for n in nstars]
    else:
        out = nstars
    # out format to in format
    if int_out:
        return out[0]
    else:
        return out


def star_factor(p, levels={.1: '`', .05 : '*', .01 : '**', .001: '***'}):
    """Create a factor with stars for a sequence of p-values

    Parameters
    ----------
    p : sequence of scalar
        Sequence of p-values.
    levels : dict {scalar: str}
        {value: star-mark} dictionary.

    Returns
    -------
    stars : Factor
        Factor with the appropriate star marking for each item in p.
    """
    sorted_levels = sorted(levels, reverse=True)
    star_labels = {i: levels[v] for i, v in enumerate(sorted_levels, 1)}
    star_labels[0] = ''
    level_values = np.reshape(sorted_levels, (-1, 1))
    stars = Factor(np.sum(p < level_values, 0), labels=star_labels)
    return stars


def oneway(Y, X, match=None, sub=None, par=True, title=None, ds=None):
    "data: for should iter over groups/treatments"
    ct = Celltable(Y, X, match=match, sub=sub, ds=ds, coercion=asvar)
    test = _oneway(ct, parametric=par)
    template = "{test}: {statistic}={value}{stars}, p={p}"
    out = template.format(**test)
    return out



def _oneway(ct, parametric=True):
    """
    Parameters
    ----------
    data:       list of groups/treatments
    parametric: bool
    within:     bool


    Returns
    -------
    dictionary with results:
    'test':      test name
    'statistic': statistic letter
    statistic:   statistic value
    'value':     "
    'p':         p value
    'stars':     stars as str

    """
    args = ct.get_data()
    if parametric:
        if ct.all_within:
            test = {'test': "NOTIMPLMENTED",
                    'statistic': 'NONE',
                    'NONE':0, 'p':0}
        else:
            test = {'test': "One-Way ANOVA",
                    'statistic': 'F'}
            test['F'], test['p'] = scipy.stats.f_oneway(*args)
    elif ct.all_within:
        test = {'test': "Friedman Chi-Square",
                'statistic': 'Q'}
        test['Q'], test['p'] = scipy.stats.friedmanchisquare(*args)
    else:
        test = {'test': "Kruskal Wallis",
                'statistic': 'H'}
        test['H'], test['p'] = scipy.stats.kruskal(*args)

    test['value'] = test[test['statistic']]
    test['stars'] = star(test['p'])
    return test



def ttest(Y, X=None, against=0, match=None, sub=None, corr='Hochberg',
           title='{desc}', ds=None):
    """T tests for one or more samples.

    parameters
    ----------
    Y : Var
        Dependent variable
    X : None | categorial
        Perform tests separately for all categories in X.
    against : scalar | str
        Baseline against which to test (scalar or category in X).
    sub : index
        Only use part of the data.
    corr : None | 'hochberg' | 'bonferroni' | 'holm'
        Method for multiple comparison correction (default 'hochberg').
    title : str
        Title for the table.
    """
    ct = Celltable(Y, X, match, sub, ds=ds, coercion=asvar)

    par = True
    if par:
        title_desc = "t-tests against %s" % against
        statistic_name = 't'
    else:
        raise NotImplementedError

    names = []; ts = []; dfs = []; ps = []

    if isinstance(against, str):
        k = len(ct.indexes) - 1
        assert against in ct.cells
        for idx in ct.indexes:
            label = ct.cells[idx]
            if against == label:
                baseline_id = idx
                baseline = ct.data[idx]

        for idx in ct.indexes:
            if idx == baseline_id:
                continue
            names.append(ct.cells[idx])
            if (ct.within is not False) and ct.within[idx, baseline_id]:
                t, p = scipy.stats.ttest_rel(baseline, ct.data[idx])
                df = len(baseline) - 1
            else:
                data = ct.data[idx]
                t, p = scipy.stats.ttest_ind(baseline, data)
                df = len(baseline) + len(data) - 2
            ts.append(t)
            dfs.append(df)
            ps.append(p)

    elif np.isscalar(against):
        k = len(ct.cells)

        for cell in ct.cells:
            label = cellname(cell)
            data = ct.data[cell].x
            t, p = scipy.stats.ttest_1samp(data, against)
            df = len(data) - 1
            names.append(label); ts.append(t); dfs.append(df); ps.append(p)

    if corr:
        ps_adjusted = mcp_adjust(ps, corr)
    else:
        ps_adjusted = np.zeros(len(ps))
    stars = star(ps, out=str)  # , levels=levels, trend=trend, corr=corr
    if len(np.unique(dfs)) == 1:
        df_in_header = True
    else:
        df_in_header = False

    table = fmtxt.Table('l' + 'r' * (3 - df_in_header + bool(corr)))
    table.title(title.format(desc=title_desc))
    if corr:
        table.caption(_get_correction_caption(corr, k))

    # header
    table.cell("Effect")
    if df_in_header:
        table.cell([statistic_name,
                    fmtxt.FMText(dfs[0], property='_'),
                    ], mat=True)
    else:
        table.cell(statistic_name, mat=True)
        table.cell('df', mat=True)
    table.cell('p', mat=True)
    if corr:
        table.cell(fmtxt.symbol('p', df=corr))
    table.midrule()

    # body
    for name, t, mark, df, p, p_adj in zip(names, ts, stars, dfs, ps, ps_adjusted):
        table.cell(name)
        tex_stars = fmtxt.Stars(mark, of=3)
        tex_t = fmtxt.FMText(t, fmt='%.2f')
        table.cell([tex_t, tex_stars])
        if not df_in_header:
            table.cell(df)

        table.cell(fmtxt.p(p))
        if corr:
            table.cell(fmtxt.p(p_adj))
    return table



def pairwise(Y, X, match=None, sub=None, ds=None,  # data in
             par=True, corr='Hochberg', trend=True,  # stats
             title='{desc}', mirror=False,  # layout
             ):
    """
    Pairwise comparison table.

    Parameters
    ----------
    Y : Var
        Dependent measure.
    X : categorial
        Categories to compare.
    match : None | Factor
        Repeated measures factor.
    sub : None | index-array
        Perform tests with a subset of the data.
    ds : None | Dataset
        If a Dataset is specified, all data-objects can be specified as
        names of Dataset variables

    """
    ct = Celltable(Y, X, match=match, sub=sub, ds=ds, coercion=asvar)
    test = _pairwise(ct.get_data(), within=ct.all_within, parametric=par,
                     corr=corr, trend=trend)

    # extract test results
    k = len(ct)
    indexes = test['pw_indexes']
    statistic = test['statistic']
    _K = test[statistic]
    _P = test['p']
    if corr:
        _Pc = mcp_adjust(_P, corr)
    _df = test['df']
    _NStars = test['stars']
    symbols = test['symbols']

    # create TABLE
    table = fmtxt.Table('l' + 'l' * (k - 1 + mirror))
    title_desc = "Pairwise {0}".format(test['test'])
    table.title(title.format(desc=title_desc))
    table.caption(test['caption'])

    # headings
    table.cell()
    cellnames = ct.cellnames()
    for name in cellnames[1 - mirror:]:
        table.cell(name)
    table.midrule()

    # tex_df = fmtxt.Element(df, "_", digits=0)
    if corr and not mirror:
        subrows = range(3)
    else:
        subrows = range(2)

    for row in range(0, k - 1 + mirror):
        for subrow in subrows:  # contains t/p
            # names column
            if subrow is 0:
                table.cell(cellnames[row])
            else:
                table.cell()
            # rows
            for col in range(1 - mirror, k):
                if row == col:
                    table.cell()
                elif col > row:
                    index = indexes[(row, col)]
                    if subrow is 0:
                        tex_cell = fmtxt.eq(statistic, _K[index], df=_df[index],
                                             stars=symbols[index],
                                             of=3 + trend)
                    elif subrow is 1:
                        tex_cell = fmtxt.eq('p', _P[index], fmt='%.3f', drop0=True)
                    elif subrow is 2:
                        tex_cell = fmtxt.eq('p', _Pc[index], df='c',
                                             fmt='%.3f', drop0=True)
                    table.cell(tex_cell)
                else:
                    if mirror and corr and subrow == 0:
                        index = indexes[(col, row)]
                        p = _Pc[index]
                        table.cell(p, fmt='%.3f', drop0=True)
                    else:
                        table.cell()
    return table



def _pairwise(data, within=True, parametric=True, corr='Hochberg',
              levels=True, trend=True):
    """
    data:   list of groups/treatments

    corr:   'Hochberg'
            'Holm'
            'Bonferroni'


    Returns
    -------
    dictionary with results, containing:

    'test': test name
    'caption': information about correction
    'statistic': abbreviation used for the staistic (e.g. 'Q')
    statistic: list of values
    'df': df
    'p': list of corresponding pa values
    'stars': list of n stars (ints)
    'pw_indexes': dict linking table index (i,j) to the list index for p etc.

    """
    # find test
    k = len(data)
    if k < 3:  # need no correction for single test
        corr = None
    if parametric:
        test_name = "t-Tests ({0} samples)"
        statistic = "t"
        if within:
            test_func = scipy.stats.ttest_rel
            test_name = test_name.format('paired')
        else:
            test_func = scipy.stats.ttest_ind
            test_name = test_name.format('independent')
    elif within:
        test_name = "Wilcoxon Signed-Rank Test"
        test_func = scipy.stats.wilcoxon
        statistic = "z"
    else:
        test_name = "Mann-Whitney U Test"
        raise NotImplementedError("mannwhitneyu returns one-sided p")
        test_func = scipy.stats.mannwhitneyu
        statistic = "u"

    # perform test
    _K = []  # kennwerte
    _P = []
    _df = []
    i = 0
    indexes = {}
    for x in range(k):
        for y in range(x + 1, k):
            Y1, Y2 = data[x], data[y]
            t, p = test_func(Y1, Y2)
            _K.append(t)
            if within:
                _df.append(len(Y1) - 1)
            else:
                _df.append(len(Y1) + len(Y2) - 2)

            _P.append(p)
            indexes[(x, y)] = indexes[(y, x)] = i
            i += 1
    # add stars
    _NStars = star(_P, out=int, levels=levels, trend=trend, corr=corr)
    _str_Stars = star(_P, out=str, levels=levels, trend=trend, corr=corr)
    caption = _get_correction_caption(corr, len(_P))
    # prepare output
    out = {'test': test_name,
           'caption': caption,
           'statistic': statistic,
           statistic: _K,
           'df': _df,
           'p': _P,
           'stars': _NStars,
           'symbols': _str_Stars,
           'pw_indexes': indexes}
    return out



def correlations(Y, Xs, cat=None, sub=None, ds=None, levels=[.05, .01, .001],
                 diff=None, pmax=None, nan=True):  # , match=None):
    """Correlation with multiple predictors.

    Parameters
    ----------
    Y : Var
        First variable
    X : Var | list of Var
        second variable (or list of variables).
    cat : categorial
        Show correlations separately for different groups in the
        data. Can be a ``Factor`` (the correlation for each level is shown
        separately) or an array of ``bool`` values (e.g. from a comparison like
        ``Stim==1``)
    levels : list of float
        Significance levels to mark.
    diff :
        (Factor, cat_1, cat_2)
    sub :
        Use only a subset of the data
    pmax : float | None
        Don't show correlations with p > pmax
    nan : bool
        Display correlation which yield NAN;
    """
    sub = assub(sub, ds)
    Y = asvar(Y, sub, ds)
    if isvar(Xs):  # FIXME: better way to specify Xs
        Xs = [Xs]
    Xs = [asvar(X, sub, ds) for X in Xs]
    if cat is not None:
        cat = ascategorial(cat, sub, ds)

    levels = np.array(levels)

    if diff is not None:
        raise NotImplementedError

    if cat is None:
        table = fmtxt.Table('l' * 4)
        table.cells('Variable', 'r', 'p', 'n')
    else:
        table = fmtxt.Table('l' * 5)
        table.cells('Variable', 'Category', 'r', 'p', 'n')

    table.midrule()
    table.title("Correlations with %s" % (Y.name))

    table._my_nan_count = 0

    for X in Xs:
        if cat is None:
            _corr_to_table(table, Y, X, cat, levels, pmax=pmax, nan=nan)
        else:
            printXname = True
            for cell in cat.cells:
                tlen = len(table)
                sub = (cat == cell)
                _corr_to_table(table, Y, X, sub, levels, pmax=pmax, nan=nan,
                               printXname=printXname, label=cellname(cell))

                if len(table) > tlen:
                    printXname = False

    # last row
    if pmax is None:
        p_text = ''
    else:
        p_text = 'all other p>{p}'.format(p=pmax)
    if nan is False and table._my_nan_count > 0:
        nan_text = '%s NANs' % table._my_nan_count
    else:
        nan_text = ''
    if p_text or nan_text:
        if p_text and nan_text:
            text = ', '.join([p_text, nan_text])
        else:
            text = ''.join([p_text, nan_text])
        table.cell("(%s)" % text)
    return table



def _corr(Y, X, sub=None):
    """
    index has to be bool array; returns r, p, n

    """
    if sub is not None:
        Y = Y[sub]
        X = X[sub]
    n = len(Y)
    assert n == len(X)
    df = n - 2
    r = np.corrcoef(Y.x, X.x)[0, 1]
    t = r / np.sqrt((1 - r ** 2) / df)
    p = scipy.stats.t.sf(np.abs(t), df) * 2
    return r, p, n



def _corr_to_table(table, Y, X, categories, levels, printXname=True, label=False,
                   pmax=None, nan=True):
    r, p, n = _corr(X, Y, categories)
    if (pmax is None) or (p <= pmax):
        if nan or (not np.isnan(r)):
            nstars = np.sum(p <= levels)
            if printXname:
                table.cell(X.name)
            else:
                table.cell()
            if label:
                table.cell(label)
            table.cells(fmtxt.FMText(r) + fmtxt.Stars(nstars, of=len(levels)), p, n)
        else:
            table._my_nan_count += 1



class bootstrap_pairwise(object):
    def __init__(self, Y, X, match=None, sub=None,
                 samples=1000, replacement=True,
                 title="Bootstrapped Pairwise Tests", ds=None):
        sub = assub(sub, ds)
        Y = asvar(Y, sub, ds)
        X = asfactor(X, sub, ds)
        assert len(Y) == len(X), "data length mismatch"
        if match is not None:
            match = ascategorial(match, sub, ds)
            assert len(match) == len(Y), "data length mismatch"

        # prepare data container
        resampled = np.empty((samples + 1, len(Y)))  # sample X subject within category
        resampled[0] = Y.x
        # fill resampled
        for i, Y_ in enumerate(resample(Y, samples, replacement, match), 1):
            resampled[i] = Y_.x
        self.resampled = resampled

        cells = X.cells
        n_groups = len(cells)

        if match is not None:
            # if there are several values per X%match cell, take the average
            # T: indexes to transform Y.x to [X%match, value]-array
            match_cell_ids = match.cells
            group_size = len(match_cell_ids)
            T = None; i = 0
            for X_cell in cells:
                for match_cell in match_cell_ids:
                    source_indexes = np.where((X == X_cell) * (match == match_cell))[0]
                    if T is None:
                        n_cells = n_groups * group_size
                        T = np.empty((n_cells, len(source_indexes)), dtype=int)
                    T[i, :] = source_indexes
                    i += 1

            if T.shape[1] == 1:
                T = T[:, 0]
                ordered = resampled[:, T]
            else:
                ordered = resampled[:, T].mean(axis=2)
            self.ordered = ordered

            # t-tests
            n_comparisons = sum(range(n_groups))
            t = np.empty((samples + 1, n_comparisons))
            comp_names = []
            one_group = np.arange(group_size)
            groups = [one_group + i * group_size for i in range(n_groups)]
            for i, (g1, g2) in enumerate(itertools.combinations(range(n_groups), 2)):
                group_1 = groups[g1]
                group_2 = groups[g2]
                diffs = ordered[:, group_1] - ordered[:, group_2]
                t[:, i] = np.mean(diffs, axis=1) * np.sqrt(group_size) / np.std(diffs, axis=1, ddof=1)
                comp_names.append(' - '.join((cells[g1], cells[g2])))

            self.diffs = diffs
            self.t_resampled = np.max(np.abs(t[1:]), axis=1)
            self.t = t = t[0]
        else:
            raise NotImplementedError

        self._Y = Y
        self._X = X
        self._group_names = cells
        self._group_data = np.array([ordered[0, g] for g in groups])
        self._group_size = group_size
        self._df = group_size - 1
        self._match = match
        self._n_samples = samples
        self._replacement = replacement
        self._comp_names = comp_names
        self._p_parametric = self.test_param(t)
        self._p_boot = self.test_boot(t)
        self.title = title

    def __repr__(self):
        out = ['bootstrap_pairwise(', self._Y.name, self._X.name]
        if self._match:
            out.append('match=%s ' % self._match.name)
        out.append('saples=%i ' % self._n_samples)
        out.append('replacement=%s)' % self._replacement)
        return ''.join(out)

    def __str__(self):
        return str(self.table())

    def table(self):
        table = fmtxt.Table('lrrrr')
        table.title(self.title)
        table.caption("Results based on %i samples" % self._n_samples)
        table.cell('Comparison')
        table.cell(fmtxt.symbol('t', df=self._df))
        table.cell(fmtxt.symbol('p', df='param'))
        table.cell(fmtxt.symbol('p', df='corr'))
        table.cell(fmtxt.symbol('p', df='boot'))
        table.midrule()

        p_corr = mcp_adjust(self._p_parametric)
        stars_parametric = star(self._p_parametric)
        stars_boot = star(self._p_boot, corr=None)

        for name, t, p1, pc, s1, p2, s2 in zip(self._comp_names, self.t,
                                           self._p_parametric, p_corr,
                                           stars_parametric,
                                           self._p_boot, stars_boot):
            table.cell(name)
            table.cell(t, fmt='%.2f')
            table.cell(fmtxt.p(p1))
            table.cell(fmtxt.p(pc, stars=s1))
            table.cell(fmtxt.p(p2, stars=s2))
        return table

    def plot_t_dist(self, ax=None):
        """
        After:
        http://stackoverflow.com/questions/4150171/how-to-create-a-density-plot-in-matplotlib
        """
        if ax is None:
            fig = P.figure()
            ax = P.axes()
        t = self.t_resampled
        density = scipy.stats.gaussian_kde(t)
#        density.covariance_factor = lambda : .25
#        density._compute_covariance()
        xs = np.linspace(0, max(t), 200)
        P.plot(xs, density(xs))

    def plot_dv_dist(self, ax=None):
        if ax is None:
            fig = P.figure()
            ax = P.axes()

        xs = np.linspace(np.min(self._group_data), np.max(self._group_data), 200)
        for i, name in enumerate(self._group_names):
            density = scipy.stats.gaussian_kde(self._group_data[i])
            P.plot(xs, density(xs), label=name)
        P.legend()

    def test_param(self, t):
        p = scipy.stats.t.sf(np.abs(t), self._group_size - 1) * 2
        return p

    def test_boot(self, t):
        "t: scalar or array; returns p for each t"
        test = self.t_resampled[:, None] > np.abs(t)
        p = np.sum(test, axis=0) / self.t_resampled.shape[0]
        return p



"""
setyup.py
=========

This is the setup.py script for Eelbrain.

To create the Alias build on Mac Os X run::

    $ python setup.py py2app -A

After that, Eelbrain.app can be found in the dist/ folder. The application
can be moved to another location without breaking the links, but will always
reflect changes to the source code in eelbrain/, which consequently has to
stay in the same location.

http://packages.python.org/py2app

http://docs.python.org/distutils/index.html

"""
from ez_setup import use_setuptools
use_setuptools()

import re
import sys
from setuptools import setup, find_packages, Extension
import numpy as np

# version must be in X.X.X format, e.g., "0.0.3dev"
with open('eelbrain/__init__.py') as fid:
    text = fid.read()
match = re.search("__version__ = '([.\w]+)'", text)
if match is None:
    raise ValueError("No valid version string found in:\n\n" + text)
version = match.group(1)
if version.count('.') != 2 and not version.endswith('dev'):
    raise ValueError("Invalid version string extracted: %r" % version)

if len(sys.argv) > 1:
    arg = sys.argv[1]
else:
    arg = None

# Cython extensions
ext = [Extension("eelbrain._stats.opt", ["eelbrain/_stats/opt.c"])]

# basic setup arguments
kwargs = dict(
              name='eelbrain',
              version=version,
              description="MEG/EEG analysis tools",
              url="https://pythonhosted.org/eelbrain",
              author="Christian Brodbeck",
              author_email='christianbrodbeck@nyu.edu',
              license='GPL3',
              long_description=open('README.txt').read(),
              install_requires=['mne >= 0.8',
                                'matplotlib >= 1.1',
                                'scipy >= 0.11.0',
                                'numpy >= 1.8',
                                'tex >= 1.8',
                                'docutils >= 0.11'],
              extras_require={'plot.brain': ['pysurfer >= 0.5',
                                             'mayavi >= 4.3.1',
                                             'nibabel >= 1.3']},
              include_dirs=[np.get_include()],
              ext_modules=ext,
              )

# py2app -----------------------------------------------------------------------
py2app_alias_script = """
# Startup script for py2app.
import os
import site
import sys

# extend sys.path
paths = %s
for path in paths:
    if path not in sys.path:
        site.addsitedir(path)


# launch Eelbrain
import eelbrain.wxterm

os.chdir(os.path.expanduser('~'))
eelbrain.wxterm.launch(True)
""" % str(sys.path)

if arg == 'py2app':
    doctypes = [
                {"CFBundleTypeExtensions": ["py"],
                 "CFBundleTypeName": "Python Script",
                 "CFBundleTypeRole": "Editor",
                 "CFBundleTypeIconFile": "icons/system-icons/pydoc.icns",
                 },
                {"CFBundleTypeExtensions": ["pickled"],
                 "CFBundleTypeName": "Pickled Python Object",
                 "CFBundleTypeRole": "Editor",
                 "CFBundleTypeIconFile": "icons/system-icons/pickled.icns",
                 },
                ]

    OPTIONS = {
               'argv_emulation': True,
               'site_packages': True,
               'iconfile': 'icons/system-icons/eelbrain.icns',
               # py2app's iconfile option places the specified file
               # in the new app bundle's Resources directory
#               'packages': 'wx',
#               'resources': ['icons/system-icons/eelbrain.icns',
#                          'resources/License.txt'
#                          ],
               'plist': dict(CFBundleName="Eelbrain",
                             CFBundleShortVersionString=version,
                             CFBundleGetInfoString="Eelbrain " + version,
                             CFBundleExecutable="Eelbrain",
                             CFBundleIdentifier="com.christianmbrodbeck.Eelbrain",
                             CFBundleDocumentTypes=doctypes,
#                             CFBundleIconFile = 'eelbrain.icns',
                             ),
               }

    # create the startup script
    if '-A' in sys.argv:
        script_path = 'scripts/eelbrain_alias.py'
        with open(script_path, 'w') as fid:
            fid.write(py2app_alias_script)
    else:
        script_path = 'scripts/eelbrain_py2app.py'

    kwargs.update(
                  app=[script_path],
                  options={'py2app': OPTIONS},
                  setup_requires=['py2app'])
# cx_freeze??? ---
# elif arg =='build':
#    from cx_Freeze import setup, Executable
#    kwargs.update(executables = [Executable("scripts/eelbrain")])
else:
    # normal & py2exe ---
    kwargs['packages'] = find_packages()
    # py2exe ---
    if arg == 'py2exe':
        # http://wiki.wxpython.org/DistributingYourApplication
        import py2exe, matplotlib
        data_files = matplotlib.get_py2exe_datafiles()

        # DLL files that various Python modules depend on
        data_files.append(('.', [
#                                 "C:\Python27\DLLs\msvcp90.dll",
#                                 "C:\Python27\lib\site-packages\wx\gdiplus.dll",
                                 "C:\python27\scripts\MK2_CORE.DLL",
                                 "C:\python27\scripts\MK2_P4P.DLL",
                                 "C:\python27\scripts\MK2IOMP5MD.DLL",
                                  ]))


        OPTIONS = dict(
#                       skip_archive = True,
#                       bundle_files = 2, # This tells py2exe to bundle everything
                       excludes=['_cairo', '_cocoaagg', '_emf', '_fltkagg',
                                 '_gtkagg', '_qt4agg', '_tkagg'],
                       includes=['matplotlib.numerix.random_array',
                                 'scipy.io.matlab.streams'],
                       dll_excludes=['libgdk-win32-2.0-0.dll',
                                     'libgobject-2.0-0.dll'])
        win_icon = "icons/system-icons/eelbrain.ico"
        kwargs.update(windows=[{"script": 'scripts/eelbrain',
                                   "icon_resources": [(0, win_icon)],
                                   }],
                      data_files=data_files,
                      options={'py2exe': OPTIONS},
                      # com_server=['myserver'],
                      )

    # normal -----------------------------------------------------------------------
    else:
        kwargs['scripts'] = ['scripts/eelbrain']


setup(**kwargs)




'''Statistical tests for multidimensional data in :class:`NDVar` objects.

.. autosummary::
   :toctree: generated

   ttest_1samp
   ttest_rel
   ttest_ind
   t_contrast_rel
   anova
   corr

In general, tests are defined as classes that provide results as attributes
and methods::

    >>> res = testnd.ttest_rel(Y, X, 'test', 'control')
    >>> res.t  # an NDVar object with a t-value for each sample

Test result objects can be directly submitted to appropriate plotting
functions. To plot only part of the results, specific attributes can be
submitted (for a description of the attributes see the relevant class
documentation)::

    >>> plot.UTS(res)  # plots values in both conditions as well as
    ... difference values with p-value thresholds
    >>> plot.UTS(res.p)  # plots only p-values


Permutation Cluster Tests
=========================

By default the tests in this module produce maps of statistical parameters
uncorrected for multiple comparison. Most tests can also form clusters and
evaluate their significance using permutation of the data [1]_.
Cluster testing is enabled by providing a ``samples`` parameter defining the
number of permutations to perform. The threshold for forming clusters is
specified as the `pmin` parameter. The default (`pmin=None`) is threshold-free
cluster enhancement [2]_.

.. warning:: Spatiotemporal permutation cluster test can take a long time to
    evaluate. It might be a good idea to estimate the time they will take using
    a very small value for ``samples`` first.


.. [1] Maris, E., & Oostenveld, R. (2007). Nonparametric
    statistical testing of EEG- and MEG-data. Journal of Neuroscience Methods,
    164(1), 177-190. doi:10.1016/j.jneumeth.2007.03.024
.. [2] Smith, S. M., and Nichols, T. E. (2009). Threshold-Free Cluster
    Enhancement: Addressing Problems of Smoothing, Threshold Dependence and
    Localisation in Cluster Inference. NeuroImage, 44(1), 83-98.
    doi:10.1016/j.neuroimage.2008.03.061

'''

__test__ = False

from ._stats.testnd import (t_contrast_rel, corr, ttest_1samp, ttest_ind,
                            ttest_rel, anova)
